:: GsplitWithVSS.bat - This script will split a large file into chunks using Volume Shadowing and portable Gsplit
:: Usage: GsplitWithVSS.bat "<inputfilespec>" "<outputFolder>" "<OS-type>" Note the output folder should NOT have a trailing /
:: Valid OS-types are: XP, 2003, 32 and 64
:: There is a package of file dependencies that accompany this script including gsplit and vshadow distributions
:: 12-Feb-2010 - Portions adapted from tech posting on the Internet - MDR - Dr.Backup
:: 21-May-2012 - Write out sentinel file if Gsplit or VSS fail - MDR - Dr.Backup
:: 10-21-2013 - If gsplit fails for any reason, REMOVE all split components already processed. Leave sentinel file only
:: 02-04-2014 - Use -nw option to skip all VSS writers in snapshot
:: 10-31-2017 - Improved error handling if no file
:: 11-28-2017 - Detect and warn if file being split is stale
:: 01-07-2020 - Gsplit run in silent (daemon) mode.
:: 11-06-2020 - Handle filenames that have embedded &

SETLOCAL

:: Customization parameters which can be adjusted
SET MIN_FILE_SIZE="300000000"
SET FREE_DRIVE_LETTER=Q:
SET CHUNK_SIZE="250"
SET CHUNK_UNITS="MB"

:: Do not change anything below this line


:: The version of VSHADOW is different on each OS. Specify it as parameter 3
:: Valid values are XP, 2003, 32 or 64 - default in build will be 32 (Vista 32 or Windows 7 -32bit)
SET OS=%3

:: If this is a callback from VSHADOW, then skip parameter setup
IF NOT "%CALLBACK_SCRIPT%"=="" GOTO :IS_CALLBACK

:: Extract the drive letter for use for the VSS Shadow, the relative path to append to the FREE_DRIVE_LETTER
:: and the destination folder to put the split files into. These parameters are all coming from the command line
SET SOURCE_DRIVE_LETTER=%~d1

:: 11/06/2020 - quote input string to handle embedded &
SET "SOURCE_RELATIVE_PATH=%~pnx1"
SET DESTINATION_PATH=%2
SET FILE_TO_COPY=%~nx1

:: We will be telling VSHADOW to do a recursive callback to this batch job, specified as parameter %0 in command line
SET CALLBACK_SCRIPT=%~dpnx0

:: Write error file here. If VSS snapshot succeeds, this will be deleted.
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(VSSError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"

:: Tell VSHADOW where to write the Volume shadow filespec after created
:: We will execute this batch script after the callback - Hardcoded the script name
SET TEMP_GENERATED_SCRIPT=GeneratedVarsTempScript.cmd

:: Lets actually create the volume shadow set now. VSHADOW will call us back when done and load definitions
:: into the TEMP_GENERATED_SCRIPT command file.
:: 02-04-2014 - skip all VSS writers. Remove -nw is writers required.
"%~dp0VSHADOW%OS%.EXE" -nw -script="%TEMP_GENERATED_SCRIPT%" -exec="%CALLBACK_SCRIPT%" %SOURCE_DRIVE_LETTER%

:: By time we get here, the callback has occured and we are done with the temporarily generated script to give us
:: the shadow file spec. Get rid of temp file
DEL /F %TEMP_GENERATED_SCRIPT%

:: We are done.
GOTO :EOF

:: Here is the magic. We are called and wind up here while the Shadow volume still exists and is mapped to a drive
:: Lets do our testing and Gsplit if file big enough

:IS_CALLBACK
SETLOCAL

:: Lets call the script created by VSHADOW to let us know the device name of the shadow volume so we can map it to a drive
:: The script should set the SHADOW_DEVICE_1 env variable
CALL %TEMP_GENERATED_SCRIPT%

:: To use the shadow volume freely, lets assign it a drive letter
:: We are using the DOSDEV utility here provided in the package
:: 8/31/2011 - remove the drive before attempting to assign
"%~dp0DOSDEV" -d -r %FREE_DRIVE_LETTER%
"%~dp0DOSDEV" %FREE_DRIVE_LETTER% %SHADOW_DEVICE_1%

:: Little housekeeping here to get rid of any previously generated gsplit files in this folder
:: We had to remove the quote marks from the original path argument if they were present
DEL /Q "%DESTINATION_PATH:"=%\GS-*.*"

:: If the file itself is not big enough, it cannot be split. Let's test first.
:: Reason we test is the gsplit will hang if fed too small a file :-(
:: Use a small shell script to do the file size testing. Included in package

START /w cscript "%~dp0Size.vbs" "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%" ">" %MIN_FILE_SIZE%

:: Return 0 = file big enough to split
IF NOT ERRORLEVEL 1 GOTO RUNGSPLIT

:: Return 1 = file not big enough to split
IF NOT ERRORLEVEL 2 GOTO TOOSMALL

:: Return 2 = file OK to split, but may be stale - WARN
IF NOT ERRORLEVEL 3 GOTO WARNSTALE

:: File does not exist or other error
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(GsplitError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"
GOTO EOF

:TOOSMALL
:: We are here because the file is too small to split. Let's just copy the file anyway
:: 12/29/2012 - Added lines to change path and "touch" file due to copy not updating MDT
COPY /Y "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%" "%DESTINATION_PATH:"=%\GS-%FILE_TO_COPY%"
cd /d "%DESTINATION_PATH:"=%"
COPY /B "%DESTINATION_PATH:"=%\GS-%FILE_TO_COPY%" +,,
GOTO :EOF

:: Possible stale file. Warn here before splitting
:WARNSTALE
ECHO VSSGSPLITSTALEWARNING-%FILE_TO_COPY%(GsplitStaleWarning) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITWARNSTALE##.TXT"

:: Well we really do have a big file to split. Let's break it up into 250MB chunks
:RUNGSPLIT

:: Write out the gsplit.gsf configuration file that tells portable gsplit what to do
Echo [MAIN]> "%~dp0gsplit.gsf"

::11/06/2020 - quote input string to handle embedded &. Cannot use -s mode due to quote parsing around string in .gsf file
Echo|set /p="Source=%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%">> "%~dp0gsplit.gsf"
Echo. >>"%~dp0gsplit.gsf"
Echo DestFolder=%DESTINATION_PATH:"=%>> "%~dp0gsplit.gsf"
Echo PieSize=%CHUNK_SIZE:"=% >> "%~dp0gsplit.gsf"
Echo PieSizeUnit=%CHUNK_UNITS:"=% >> "%~dp0gsplit.gsf"
Echo PieMask=GS-{ofw}.{oru} >> "%~dp0gsplit.gsf"
Echo SplitMethod=0 >> "%~dp0gsplit.gsf"
Echo TypePieSpanned=0 >> "%~dp0gsplit.gsf"
Echo Title=Gsplit-Backup >> "%~dp0gsplit.gsf"

:: Now that everything is setup, we can use the portable gsplit utility
:: 11/06/2020 - Removed -s mode on gsplit to accomodate &
"%~dp0gsplit.exe" "no" "-b-i-t-s" "%~dp0gsplit.gsf"

:: Test for gsplit error here and write out sentinel file if needed
:: 10/21/2013 - Remove ALL split components if Gsplit doesn't return success
IF %ERRORLEVEL% EQU 0 GOTO :EOF
DEL /Q "%DESTINATION_PATH:"=%\GS-*.*"
ECHO VSSGSPLITERROR-%FILE_TO_COPY%(GsplitError) > "%DESTINATION_PATH:"=%\GS-##VSSGSPLITERROR##.TXT"

:EOF
:: All done here